import { WalkNode } from "./node";
export declare type Context<T> = {
    config: Config<T>;
    nodes: any;
    seenObjects: any[];
    callbacksByPosition: {
        [key: string]: BaseCallback[];
    };
};
export declare type NodeType = 'array' | 'object' | 'value';
export declare type GraphMode = 'finiteTree' | 'tree' | 'graph' | 'infinite';
export declare type PositionType = 'preWalk' | 'postWalk';
export declare type TraversalMode = 'depth' | 'breadth';
export interface IOrderable {
    executionOrder?: number;
}
export declare type Cb = (node: WalkNode) => void;
export declare type AsyncCb = Cb | ((node: WalkNode) => Promise<void>);
export declare type BaseCallback = IOrderable & {
    executionOrder?: number;
    positionFilters?: PositionType[];
    keyFilters?: string[];
    nodeTypeFilters?: NodeType[];
};
export declare type Callback = BaseCallback & {
    callback: Cb;
};
export declare type AsyncCallback = BaseCallback & {
    callback: AsyncCb;
};
export declare type Config<T extends BaseCallback> = {
    readonly traversalMode: TraversalMode;
    readonly callbacks: T[];
    readonly graphMode: GraphMode;
    readonly rootObjectCallbacks: boolean;
    readonly runCallbacks: boolean;
    readonly parallelizeAsyncCallbacks: boolean;
};
export declare type PartialConfig<T extends BaseCallback> = {
    traversalMode?: TraversalMode;
    callbacks?: T[];
    graphMode?: GraphMode;
    rootObjectCallbacks?: boolean;
    runCallbacks?: boolean;
    parallelizeAsyncCallbacks?: boolean;
};
