"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Break = exports.deepCopy = exports.findAll = exports.applyAsync = exports.apply = exports.flatten = void 0;
const helpers_1 = require("./helpers");
const walk_1 = require("./walk");
function flatten(obj, key, onlyUnique) {
    //return array of values that match the key
    const arr = [];
    walk_1.walk(obj, {
        callbacks: [{
                keyFilters: [key],
                callback: function (node) {
                    arr.push(node.val);
                }
            }]
    });
    return onlyUnique ? helpers_1.unique(arr) : arr;
}
exports.flatten = flatten;
function apply(obj, ...callbacks) {
    walk_1.walk(obj, { callbacks: callbacks.map(c => ({ callback: c })) });
}
exports.apply = apply;
function applyAsync(obj, ...callbacks) {
    return __awaiter(this, void 0, void 0, function* () {
        yield walk_1.walkAsync(obj, { callbacks: callbacks.map(c => ({ callback: c })) });
    });
}
exports.applyAsync = applyAsync;
function findAll(obj, value, typeConversion = false) {
    const comparison = typeConversion
        ? (a, b) => a == b
        : (a, b) => a === b;
    const matches = [];
    apply(obj, function (node) {
        if (comparison(node.val, value)) {
            matches.push(node.val);
        }
    });
    return matches;
}
exports.findAll = findAll;
function deepCopy(obj) {
    const newObj = {};
    const uuid = 'WALK:DEEP-COPY:DELIMITER';
    const format = (key) => uuid + key;
    walk_1.walk(obj, {
        rootObjectCallbacks: false,
        callbacks: [{
                positionFilters: ['preWalk'],
                callback: function (node) {
                    switch (node.nodeType) {
                        case 'array':
                            helpers_1.updateObjectViaPathString(newObj, [], node.getPath(format), uuid);
                            break;
                        case 'object':
                            helpers_1.updateObjectViaPathString(newObj, {}, node.getPath(format), uuid);
                            break;
                        case 'value':
                            helpers_1.updateObjectViaPathString(newObj, node.val, node.getPath(format), uuid);
                            break;
                    }
                }
            }]
    });
    return newObj;
}
exports.deepCopy = deepCopy;
class Break extends Error {
    constructor(message = '') {
        super(message);
        this.name = "Break";
    }
}
exports.Break = Break;
